import { __assign } from 'tslib';
import { useState, useRef, useCallback, useEffect } from 'react';
import { createRichTextEditor } from '../createRichTextEditor.mjs';

/**
 * useRichTextEditor
 * React hook to initialize a ProseMirror editor, handle binding it to the DOM,
 * and updating the state within React’s lifecycle.
 *
 * @param {initialEditorState} ProseMirror state
 * @returns {Array}
 */
var useRichTextEditor = function (initialEditorState,
/*
 * Pass in HTML attributes into the parent RTE node
 */
attributes, options) {
  options = __assign({
    editable: true
  }, options);
  var _a = useState(initialEditorState),
    editorState = _a[0],
    setEditorState = _a[1];
  // Refs to hold the methods returned from ProseMirror’s initialization
  var destroyEditorRef = useRef();
  var dispatchTransactionRef = useRef(function () {
    return undefined;
  });
  // Construct a consistent reference to call the dispatchTransactionRef without
  // forcing the consumer to unwind it
  var dispatchTransaction = useCallback(function (commandOrTransaction) {
    dispatchTransactionRef.current(commandOrTransaction);
  }, [dispatchTransactionRef]);
  // Hold editableStatus as a ref so we can toggle its status
  var editableStatusRef = useRef(options.editable);
  var setEditableStatus = useCallback(function (status) {
    editableStatusRef.current = status;
    // Trigger an update within ProseMirror by issuing a noop transaction
    dispatchTransaction(function (state, dispatch) {
      if (!dispatch) return false;
      dispatch(state.tr);
      return true;
    });
  },
  // @todo: Fix if possible - avoiding breaking in eslint upgrade
  // eslint-disable-next-line react-hooks/exhaustive-deps
  [editableStatusRef]);
  var editorRef = useCallback(function (node) {
    if (node !== null) {
      var instance = createRichTextEditor({
        node: node,
        initialEditorState: editorState,
        onChange: setEditorState,
        isEditable: function () {
          return editableStatusRef.current;
        },
        attributes: attributes
      });
      destroyEditorRef.current = instance.destroy;
      dispatchTransactionRef.current = instance.dispatchTransaction;
    }
  },
  // Including editorState in the dependencies here will cause an endless
  // loop as the initialization changes its value
  // @todo: Fix if possible - avoiding breaking in eslint upgrade
  // eslint-disable-next-line react-hooks/exhaustive-deps
  [setEditorState, editableStatusRef]);
  // Tear down ProseMirror when the consuming component is unmounted
  useEffect(function () {
    return function () {
      if (destroyEditorRef.current) {
        destroyEditorRef.current();
      }
    };
  }, [destroyEditorRef]);
  return [editorRef, editorState, dispatchTransaction, setEditableStatus];
};
export { useRichTextEditor };
