import debounce from 'lodash.debounce';
import { Plugin } from 'prosemirror-state';
import { getMarkAttrs } from '../../commands/getMarkAttrs.mjs';
import { getMarkRange } from '../../commands/getMarkRange.mjs';
import 'prosemirror-utils';
import { markContainsSelection } from '../../commands/markContainsSelection.mjs';
import { markIsActive } from '../../commands/markIsActive.mjs';
import { removeMark } from '../../commands/removeMark.mjs';
import { updateMark } from '../../commands/updateMark.mjs';
import { validateAndRemoveMarks } from '../../commands/validateAndRemoveMarks.mjs';
import { LinkEditor } from './components/LinkEditor/LinkEditor.mjs';
import { createReactTooltipWrapper } from './createReactTooltipWrapper.mjs';
var LinkManager = /** @class */function () {
  function LinkManager(view, markType, editorComponent, linkAttributeValidator) {
    var _this = this;
    this.editorComponent = editorComponent;
    this.linkActive = function (state) {
      return markIsActive(state, markType);
    };
    this.validateLinks = validateAndRemoveMarks(markType, linkAttributeValidator);
    this.markType = markType;
    this.tooltipTarget = null;
    this.update(view, null);
    this.onResize = debounce(function () {
      _this.update(view, null);
    }, 15);
  }
  LinkManager.prototype.update = function (view, lastState) {
    var state = view.state;
    // Don’t do anything if the document/TextSelection didn't change
    if (lastState && lastState.doc.eq(state.doc) && lastState.selection.eq(state.selection)) {
      return;
    }
    // Is the selection empty or not a link
    //
    // TODO: Also need to check if the selection is _only_ a link. If it covers
    // more than a single mark then we don’t want to allow edited
    if (state.selection.empty || !this.linkActive(state)) {
      this.destroyElement(view);
      return;
    }
    // If there is a selection and we dont have something in DOM
    // then create element and inject
    if (!this.tooltipTarget) {
      this.createElement(view);
    }
    this.updateElement(view);
  };
  LinkManager.prototype.destroy = function () {
    var _a;
    (_a = this.tooltipTarget) === null || _a === void 0 ? void 0 : _a.destroy();
    window.removeEventListener('resize', this.onResize);
  };
  LinkManager.prototype.createElement = function (view) {
    if (!view.dom.parentElement) return;
    this.tooltipTarget = createReactTooltipWrapper(view.dom.parentElement, this.editorComponent, this.getEditorProps(view));
    window.addEventListener('resize', this.onResize);
  };
  LinkManager.prototype.updateElement = function (view) {
    var _a;
    (_a = this.tooltipTarget) === null || _a === void 0 ? void 0 : _a.update(this.getEditorProps(view));
  };
  LinkManager.prototype.destroyElement = function (view) {
    var _a;
    view.dispatch(this.validateLinks);
    (_a = this.tooltipTarget) === null || _a === void 0 ? void 0 : _a.destroy();
    this.tooltipTarget = null;
  };
  LinkManager.prototype.getEditorProps = function (view) {
    var _this = this;
    var selectionPosition = null;
    var selection = view.state.selection;
    var contained = markContainsSelection(view.state, this.markType);
    var $from = selection.$from;
    var range = getMarkRange($from, this.markType);
    var start;
    var end;
    if (!contained && range) {
      start = view.coordsAtPos(range.from);
      end = view.coordsAtPos(range.to);
    } else {
      start = view.coordsAtPos(selection.from);
      if (selection.to === view.state.doc.content.size) {
        end = view.coordsAtPos(selection.to - 1);
      } else {
        end = view.coordsAtPos(selection.to);
      }
    }
    selectionPosition = {
      top: start.top,
      left: start.left,
      width: end.right - start.left,
      height: start.top - start.bottom
    };
    return {
      removeMark: function () {
        view.dispatch(removeMark(_this.markType, {
          toExtent: true
        }));
      },
      updateAttrs: function (attrs) {
        view.dispatch(updateMark(_this.markType, attrs, {
          toExtent: true
        }));
      },
      attrs: getMarkAttrs(view.state, this.markType),
      selectionPosition: selectionPosition,
      focusEditor: function () {
        view.focus();
      }
    };
  };
  return LinkManager;
}();
var defaultLinkAttributeValidator = function (attrs) {
  if (attrs == null) {
    return false;
  }
  return attrs.href && attrs.href !== '';
};
var createLinkManager = function (_a) {
  var markType = _a.markType,
    _b = _a.editorComponent,
    editorComponent = _b === void 0 ? LinkEditor : _b,
    _c = _a.linkAttributeValidator,
    linkAttributeValidator = _c === void 0 ? defaultLinkAttributeValidator : _c;
  return new Plugin({
    view: function (editorView) {
      return new LinkManager(editorView, markType, editorComponent, linkAttributeValidator);
    }
  });
};
export { createLinkManager };
